@@ -2,7 +2,7 @@ from .trajectory import graph, trajectory, trajectory_3d
 from .features import trends, single_trend
 from .test_association import test_association
 from .modules import modules
-from .milestones import milestones
+from .milestones import milestones, milestones_graph
 from .slide_cors import slide_cors
 from .synchro_path import synchro_path
 from .test_fork import test_fork
@@ -10,3 +10,5 @@ from .module_inclusion import module_inclusion
 from .linearity_deviation import linearity_deviation
 from .dendrogram import dendrogram
 from .binned_pseudotime_meta import binned_pseudotime_meta
+from .matrix import matrix
+from .covariate import trend_covariate
