@@ -13,6 +13,7 @@ from ..tools.utils import getpath
 from .trajectory import trajectory as plot_trajectory
 from .utils import setup_axes
 from ..tools.graph_operations import subset_tree
+from .milestones import milestones as milestones_plot
 from ..get import modules as get_modules
 from .. import settings
 
@@ -85,6 +86,12 @@ def modules(
         adata_c = subset_tree(adata, root_milestone, milestones, copy=True)
         adata_c.obsm["X_early"] = X_early.loc[adata_c.obs_names].values
         adata_c.obsm["X_late"] = X_late.loc[adata_c.obs_names].values
+        adata_c.uns["seg_colors"] = [
+            np.array(adata_c.uns["milestones_colors"])[
+                pd.Series(adata_c.uns["graph"]["milestones"]) == t
+            ][0]
+            for t in adata_c.uns["graph"]["pp_seg"].to
+        ]
     else:
         adata_c = AnnData(
             X_early.values,
@@ -120,21 +127,21 @@ def modules(
             axs, _, _, _ = setup_axes(panels=[0])
             ax_late = axs[0]
 
-    color = "old_milestones" if ((color == "milestones") & (nmil > 4)) else color
+    if (color == "milestones") & ("old_milestones" in adata_c.obs):
+        color = "old_milestones"
+
+    if (color == "seg") & ("old_seg" in adata_c.obs):
+        color = "old_seg"
 
     if (module == "early") | (module == "all"):
-        if show_traj:
-            plot_trajectory(
+        if color == "old_milestones":
+            milestones_plot(
                 adata_c,
                 basis="early",
-                root_milestone=root_milestone,
-                milestones=milestones,
-                color_cells=color,
-                show=False,
+                subset=cells,
                 title="",
-                legend_loc="none",
+                show=False,
                 ax=ax_early,
-                **kwargs,
             )
         else:
             sc.pl.embedding(
@@ -147,20 +154,44 @@ def modules(
                 ax=ax_early,
                 **kwargs,
             )
+        if show_traj:
+            if color != "milestones":
+                kwargs["legend_loc"] = "none"
+            plot_trajectory(
+                adata_c,
+                basis="early",
+                root_milestone=root_milestone,
+                milestones=milestones,
+                alpha=0,
+                show=False,
+                title="",
+                ax=ax_early,
+                **kwargs,
+            )
         ax_early.set_xlabel("early " + milestones[0])
         ax_early.set_ylabel("early " + milestones[1])
 
     if (module == "late") | (module == "all"):
-        sc.pl.embedding(
-            adata_c[cells],
-            basis="late",
-            color=color,
-            legend_loc="none",
-            show=False,
-            title="",
-            ax=ax_late,
-            **kwargs,
-        )
+        if color == "old_milestones":
+            milestones_plot(
+                adata_c,
+                basis="late",
+                subset=cells,
+                title="",
+                show=False,
+                ax=ax_late,
+            )
+        else:
+            sc.pl.embedding(
+                adata_c[cells],
+                basis="late",
+                color=color,
+                legend_loc="none",
+                show=False,
+                title="",
+                ax=ax_late,
+                **kwargs,
+            )
         ax_late.set_xlabel("late " + milestones[0])
         ax_late.set_ylabel("late " + milestones[1])
 
