@@ -126,7 +126,10 @@ def synchro_path(
         logg.info("    single mapping")
 
     def synchro_map(m):
-        df = adata.uns["pseudotime_list"][str(m)]
+        if n_map == 1:
+            df = adata.obs.loc[:, ["t", "seg"]]
+        else:
+            df = adata.uns["pseudotime_list"][str(m)]
         edges = graph["pp_seg"][["from", "to"]].astype(str).apply(tuple, axis=1).values
         img = igraph.Graph()
         img.add_vertices(
@@ -259,7 +262,10 @@ def synchro_path(
 
     adata.obs["inter_cor " + name] = list(map(inter_values, tval))
 
-    df = adata.uns["pseudotime_list"][str(0)]
+    if n_map == 1:
+        df = adata.obs.loc[:, ["t", "seg"]]
+    else:
+        df = adata.uns["pseudotime_list"][str(0)]
     cells = np.concatenate(
         [
             getpath(img, root, graph["tips"], leaves[0], graph, df).index,
@@ -396,8 +402,8 @@ def module_inclusion(
     milestones,
     w: int = 300,
     step: int = 30,
-    pseudotime_offset: Union["all", float] = "all",
-    module: Literal["all", "early"] = "all",
+    pseudotime_offset: Union["all", float] = 0,
+    module: Literal["all", "early"] = "early",
     n_perm: int = 10,
     n_map: int = 1,
     map_cutoff: float = 0.8,
@@ -406,7 +412,7 @@ def module_inclusion(
     autocor_cut: float = 0.95,
     iterations: int = 15,
     parallel_mode: Literal["window", "mappings"] = "window",
-    identify_early_features: bool = True,
+    identify_early_features: bool = False,
     layer=None,
     perm: bool = False,
     winp: int = 10,
@@ -511,7 +517,10 @@ def module_inclusion(
     fork_t = adata.uns["graph"]["pp_info"].loc[common_seg, "time"].max()
 
     def onset_map(m):
-        df = adata.uns["pseudotime_list"][str(m)]
+        if n_map == 1:
+            df = adata.obs.loc[:, ["t", "seg"]]
+        else:
+            df = adata.uns["pseudotime_list"][str(m)]
         edges = graph["pp_seg"][["from", "to"]].astype(str).apply(tuple, axis=1).values
         img = igraph.Graph()
         img.add_vertices(
@@ -526,9 +535,7 @@ def module_inclusion(
                 if module == "early"
                 else sel
             )
-            geneset = adata.uns[name]["fork"].index[
-                adata.uns[name]["fork"].branch == milestone
-            ]
+            geneset = adata.uns[name]["fork"].index[sel]
             cells = getpath(
                 img,
                 root,
@@ -700,6 +707,9 @@ def module_inclusion(
         props = 1 - np.isnan(matSwitch[m]).sum(axis=1) / n_map
         matSwitch[m] = matSwitch[m].loc[props > map_cutoff]
 
+    adata.uns[name]["fork"]["props_incl"] = np.nan
+    adata.uns[name]["fork"].loc[props.index, "props_incl"] = props.values
+
     adata.uns[name]["module_inclusion" + perm_str] = matSwitch
 
     updated = "."
