@@ -6,6 +6,7 @@ from joblib import delayed
 from tqdm import tqdm
 import sys
 import igraph
+from typing import Optional, Union
 
 from .utils import ProgressParallel
 
@@ -13,7 +14,13 @@ from .. import logging as logg
 from .. import settings
 
 
-def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = False):
+def pseudotime(
+    adata: AnnData,
+    n_jobs: int = 1,
+    n_map: int = 1,
+    seed: Optional[int] = None,
+    copy: bool = False,
+):
     """\
     Compute pseudotime.
 
@@ -27,6 +34,8 @@ def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = Fal
         Number of cpu processes to use in case of performing multiple mapping.
     n_map
         number of probabilistic mapping of cells onto the tree to use. If n_map=1 then likelihood cell mapping is used.
+    seed
+        A numpy random seed for reproducibility for muliple mappings
     copy
         Return a copy instead of writing to adata.
     Returns
@@ -54,6 +63,8 @@ def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = Fal
     adata = adata.copy() if copy else adata
 
     graph = adata.uns["graph"]
+    pp_seg = adata.uns["graph"]["pp_seg"]
+    pp_info = adata.uns["graph"]["pp_info"]
 
     reassign, recolor = False, False
     if "milestones" in adata.obs:
@@ -66,13 +77,24 @@ def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = Fal
 
     logg.info("projecting cells onto the principal graph", reset=True)
 
+    from sklearn.metrics import pairwise_distances
+
+    P = pairwise_distances(graph["F"].T, metric=graph["metrics"])
+
     if n_map == 1:
-        df_l = [map_cells(graph, R=adata.obsm["X_R"], multi=False)]
+        df_l = [map_cells(graph, R=adata.obsm["X_R"], P=P, multi=False)]
     else:
+        if seed is not None:
+            np.random.seed(seed)
+            map_seeds = np.random.randint(999999999, size=n_map)
+        else:
+            map_seeds = [None for i in range(n_map)]
         df_l = ProgressParallel(
             n_jobs=n_jobs, total=n_map, file=sys.stdout, desc="    mappings"
         )(
-            delayed(map_cells)(graph=graph, R=adata.obsm["X_R"], multi=True)
+            delayed(map_cells)(
+                graph=graph, R=adata.obsm["X_R"], P=P, multi=True, map_seed=map_seeds[m]
+            )
             for m in range(n_map)
         )
 
@@ -117,20 +139,90 @@ def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = Fal
             .values
         )
 
-    milestones = pd.Series(index=adata.obs_names)
-    for seg in graph["pp_seg"].n:
+        # reassign cells to their closest segment
+        root = adata.uns["graph"]["root"]
+        tips = adata.uns["graph"]["tips"]
+        endpoints = tips[tips != root]
+
+        allsegs = pd.concat(
+            [df.seg for df in adata.uns["pseudotime_list"].values()], axis=1
+        )
+        allsegs = allsegs.apply(lambda x: x.value_counts(), axis=1)
+        adata.obs.seg = allsegs.idxmax(axis=1)
+        adata.obs.t = pd.concat(
+            [df.t for df in adata.uns["pseudotime_list"].values()], axis=1
+        ).mean(axis=1)
+
+        for s in pp_seg.n:
+            df_seg = adata.obs.loc[adata.obs.seg == s, "t"]
+
+            # reassign cells below minimum pseudotime of their assigned seg
+            if any(int(s) == pp_seg.index[pp_seg["from"] != root]):
+                start_t = pp_info.loc[pp_seg["from"], "time"].iloc[int(s) - 1]
+                cells_back = allsegs.loc[df_seg[df_seg < start_t].index]
+                ncells = cells_back.shape[0]
+                if ncells != 0:
+                    filter_from = pd.concat(
+                        [pp_info.loc[pp_seg["from"], "time"] for i in range(ncells)],
+                        axis=1,
+                    ).T.values
+                    filter_to = pd.concat(
+                        [pp_info.loc[pp_seg["to"], "time"] for i in range(ncells)],
+                        axis=1,
+                    ).T.values
+                    t_cells = adata.obs.loc[cells_back.index, "t"]
+
+                    boo = (filter_from < t_cells.values.reshape((-1, 1))) & (
+                        filter_to > t_cells.values.reshape((-1, 1))
+                    )
+
+                    cells_back = (cells_back.fillna(0) * boo).apply(
+                        lambda x: x.index[np.argsort(x)][::-1], axis=1
+                    )
+                    cells_back = cells_back.apply(lambda x: x[x != s][0])
+                    adata.obs.loc[cells_back.index, "seg"] = cells_back.values
+
+            # reassign cells over maximum pseudotime of their assigned seg
+            if any(int(s) == pp_seg.index[~pp_seg.to.isin(endpoints)]):
+                end_t = pp_info.loc[pp_seg["to"], "time"].iloc[int(s) - 1]
+                cells_front = allsegs.loc[df_seg[df_seg > end_t].index]
+                ncells = cells_front.shape[0]
+                if ncells != 0:
+                    filter_from = pd.concat(
+                        [pp_info.loc[pp_seg["from"], "time"] for i in range(ncells)],
+                        axis=1,
+                    ).T.values
+                    filter_to = pd.concat(
+                        [pp_info.loc[pp_seg["to"], "time"] for i in range(ncells)],
+                        axis=1,
+                    ).T.values
+
+                    t_cells = adata.obs.loc[cells_front.index, "t"]
+
+                    boo = (filter_to > t_cells.values.reshape((-1, 1))) & (
+                        filter_from < t_cells.values.reshape((-1, 1))
+                    )
+
+                    cells_front = (cells_front.fillna(0) * boo).apply(
+                        lambda x: x.index[np.argsort(x)][::-1], axis=1
+                    )
+                    cells_front = cells_front.apply(lambda x: x[x != s][0])
+                    adata.obs.loc[cells_front.index, "seg"] = cells_front.values
+
+    milestones = pd.Series(index=adata.obs_names, dtype=str)
+    for seg in pp_seg.n:
         cell_seg = adata.obs.loc[adata.obs["seg"] == seg, "t"]
         if len(cell_seg) > 0:
             milestones[
                 cell_seg.index[
                     (cell_seg - min(cell_seg) - (max(cell_seg - min(cell_seg)) / 2) < 0)
                 ]
-            ] = graph["pp_seg"].loc[int(seg), "from"]
+            ] = pp_seg.loc[int(seg), "from"]
             milestones[
                 cell_seg.index[
                     (cell_seg - min(cell_seg) - (max(cell_seg - min(cell_seg)) / 2) > 0)
                 ]
-            ] = graph["pp_seg"].loc[int(seg), "to"]
+            ] = pp_seg.loc[int(seg), "to"]
     adata.obs["milestones"] = milestones
     adata.obs.milestones = (
         adata.obs.milestones.astype(int).astype("str").astype("category")
@@ -143,6 +235,10 @@ def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = Fal
         )
     )
 
+    # setting consistent color palettes
+    from ..plot import palette_tools
+
+    palette_tools._set_default_colors_for_categorical_obs(adata, "milestones")
     while reassign:
         if "tmp_mil_col" not in locals():
             break
@@ -152,6 +248,12 @@ def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = Fal
         if recolor:
             adata.uns["milestones_colors"] = tmp_mil_col
         reassign = False
+    adata.uns["seg_colors"] = [
+        np.array(adata.uns["milestones_colors"])[
+            pd.Series(adata.uns["graph"]["milestones"]) == t
+        ][0]
+        for t in adata.uns["graph"]["pp_seg"].to
+    ]
 
     logg.info("    finished", time=True, end=" " if settings.verbosity > 2 else "\n")
     logg.hint(
@@ -166,13 +268,15 @@ def pseudotime(adata: AnnData, n_jobs: int = 1, n_map: int = 1, copy: bool = Fal
     return adata if copy else None
 
 
-def map_cells(graph, R, multi=False):
+def map_cells(graph, R, P, multi=False, map_seed=None):
     import igraph
 
     g = igraph.Graph.Adjacency((graph["B"] > 0).tolist(), mode="undirected")
     # Add edge weights and node labels.
-    g.es["weight"] = graph["B"][graph["B"].nonzero()]
+    g.es["weight"] = np.array(P[graph["B"].nonzero()].ravel()).tolist()[0]
+
     if multi:
+        np.random.seed(map_seed)
         rrm = (
             np.apply_along_axis(
                 lambda x: np.random.choice(np.arange(len(x)), size=1, p=x),
@@ -187,16 +291,15 @@ def map_cells(graph, R, multi=False):
         vcells = np.argwhere(rrm == v)
 
         if vcells.shape[0] > 0:
-            nv = np.array(g.neighborhood(v, order=1))
-            nvd = np.array(g.shortest_paths(v, nv)[0])
+            nv = np.array(g.neighborhood(v, order=1))[1:]
+            nvd = g.shortest_paths(v, nv, weights=g.es["weight"])
 
-            spi = np.apply_along_axis(np.argmax, axis=1, arr=R[vcells, nv[1:]])
             ndf = pd.DataFrame(
                 {
                     "cell": vcells.flatten(),
                     "v0": v,
-                    "v1": nv[1:][spi],
-                    "d": nvd[1:][spi],
+                    "v1": nv[np.argmin(nvd)],
+                    "d": np.min(nvd),
                 }
             )
 
@@ -205,18 +308,14 @@ def map_cells(graph, R, multi=False):
                 list(map(lambda x: R[vcells[x], ndf.v1[x]], range(len(vcells))))
             ).flatten()
 
-            alpha = np.random.uniform(size=len(vcells))
-            f = np.abs(
-                (np.sqrt(alpha * p1 ** 2 + (1 - alpha) * p0 ** 2) - p0) / (p1 - p0)
-            )
-            ndf["t"] = (
-                graph["pp_info"].loc[ndf.v0, "time"].values
-                + (
-                    graph["pp_info"].loc[ndf.v1, "time"].values
-                    - graph["pp_info"].loc[ndf.v0, "time"].values
+            ndf["t"] = [
+                np.average(
+                    graph["pp_info"].time[ndf.iloc[i, [1, 2]].astype(int)],
+                    weights=[p0[i], p1[i]],
                 )
-                * alpha
-            )
+                for i in range(ndf.shape[0])
+            ]
+
             ndf["seg"] = 0
             isinfork = (graph["pp_info"].loc[ndf.v0, "PP"].isin(graph["forks"])).values
             ndf.loc[isinfork, "seg"] = (
@@ -242,13 +341,25 @@ def map_cells(graph, R, multi=False):
     return df
 
 
-def rename_milestones(adata, new, copy: bool = False):
+def rename_milestones(adata, new: Union[list, dict], copy: bool = False):
 
     adata = adata.copy() if copy else adata
 
-    adata.uns["graph"]["milestones"] = dict(
-        zip(new, list(adata.uns["graph"]["milestones"].values()))
-    )
+    if isinstance(new, dict):
+        old = pd.Series(
+            adata.uns["graph"]["milestones"].keys(),
+            index=adata.uns["graph"]["milestones"].values(),
+        )
+        dct = pd.Series(new)
+        dct.index = dct.index.astype(int)
+        adata.uns["graph"]["milestones"] = dict(
+            zip(dct.loc[old.index].values, adata.uns["graph"]["milestones"].values())
+        )
+        new = dct.loc[old.index].values
+    else:
+        adata.uns["graph"]["milestones"] = dict(
+            zip(new, list(adata.uns["graph"]["milestones"].values()))
+        )
 
     adata.obs.milestones = adata.obs.milestones.cat.rename_categories(new)
 
